// ppcbf.h - description.  -*- C++ -*-

// Copyright (C) 2002 Red Hat, Inc.
// Written by Johan Rydberg, jrydberg@rtmk.org

// This file is part of SID and is licensed under the GPL.
// See the file COPYING.SID for conditions for redistribution.

#include "cgen-cpu.h"
#include "ppcbf-desc.h"
#include "ppcbf-defs.h"
#include "ppcbf-decode.h"
#include <stdio.h>

namespace ppcbf 
{
  // ??? this is not a good thing (tm).
#define BITTEST(T, V, B) EXTRACT_MSB0_UINT (V, T, B, 1)

  // ??? These should really be implemented in CGEN RTL.
#define _MASKn(WIDTH, START, STOP)		\
	(((((unsigned int)0) - 1)		\
	  >> (WIDTH - ((STOP) - (START) + 1)))	\
	  << (WIDTH - 1 - (STOP)))

#define MASK(START, STOP)					\
	(((START) <= (STOP)) 					\
	 ? (((STOP) < 32) 					\
	    ? 0 						\
	    : _MASKn(32, 					\
	             (START) < 32 ? 0 : (START) - 32, 		\
	             (STOP)-32)) 				\
	 : (_MASKn(32, 						\
	           (START) < 32 ? 0 : (START) - 32, 		\
	           31) 						\
	    | (((STOP) < 32) 					\
	       ? 0 						\
	       : _MASKn(32, 					\
	                0, 					\
	                (STOP) - 32))))

class ppcbf_cpu_cgen
{
// Include cgen generated elements.
#include "ppcbf-cpu.h"

  // ??? These should really be implemented in CGEN RTL.
  inline INT ppcbf_cti_resolv_ctr (UINT bo, UINT bi);
  inline INT ppcbf_cti_resolv_cond (UINT bo, UINT bi);
  inline INT ppcbf_get_reg_or_zero (UINT regno);
};

inline INT
ppcbf_cpu_cgen::ppcbf_cti_resolv_ctr (UINT bo, UINT bi)
{
  return (BITTEST (5, bo, 2) || ((h_ctr_get () != 0) != BITTEST (5, bo, 3)));
}

inline INT
ppcbf_cpu_cgen::ppcbf_cti_resolv_cond (UINT bo, UINT bi)
{
  return (BITTEST (5, bo, 0) || (BITTEST (32, h_cr_get (), bi) == BITTEST (5, bo, 1)));
}

inline INT
ppcbf_cpu_cgen::ppcbf_get_reg_or_zero (UINT regno)
{
  if (regno != 0)
    return h_gpr_get (regno);
  return 0;
}

class ppcbf_cpu: public ppcbf_cpu_cgen, public cgen_bi_endian_cpu
{
private:
  scache_engine<ppc_scache> engine;

  void step_insns ();
  void flush_icache ();
  void reset ();

  // debug support routines
  string dbg_get_reg (host_int_4 n);
  component::status dbg_set_reg (host_int_4 n, const string& s);

  // overload state save & restore
  void stream_state(ostream& o) const;
  void destream_state(istream& i);

public:
  ppcbf_cpu ();
  ~ppcbf_cpu () throw() { };

  void set_pc (host_int_4 v) 
    { 
      this->hardware.h_pc = (PCADDR) v; 
    }

  // Called by semantic code to perform branches.
  inline void
  branch (PCADDR new_pc, PCADDR& npc, sem_status& status)
    {
      npc = new_pc;
    }

  // Called by the semantic code at the end of a non-cti insn.
  inline void
  done_insn (PCADDR npc, sem_status& status)
    {
      this->h_pc_set (npc);
    }

  // Called by the semantic code at the end of a cti insn.
  inline void
  done_cti_insn (PCADDR npc, sem_status& status)
    {
      this->h_pc_set (npc);
    }

  SI ppcbf_trap (PCADDR pc, UINT trap);
  SI ppcbf_rfi (void);

  void invalid_insn (PCADDR pc);
  void memory_trap (const cpu_memory_fault& t);
};



} // namespace ppcbf
