// ppc.h - Class declaration for the PPC gloss component, using
// Cygnus' libgloss.  -*- C++ -*-

// Copyright (C) 2002 Red Hat, Inc.
// Written by Johan Rydberg, jrydberg@rtmk.org

// This file is part of SID and is licensed under the GPL.
// See the file COPYING.SID for conditions for redistribution.

#ifndef PPC_H
#define PPC_H

#include "gloss.h"

class ppc_libgloss: public gloss32
{
public:

  ppc_libgloss();

private:

  // ABI-specifics, for getting syscall arguments and setting results.
  // ??? Class by itself.
  // ??? Perhaps this shouldn't be here at all as it's really an
  // implementation detail (but lots of implementations will probably use it).
  bool get_int_argument(unsigned index, int32& value);
  bool set_int_result(int32 value);
  bool set_error_result(int32 value);

  // Return boolean indicating if cpu is requesting something we're to handle.
  bool syscall_trap_p();
  // Perform a system call trap.
  void syscall_trap();

  // Convert host errno to target errno.
  int host_to_target_errno (int errno_);
};

class ppc_ppcbug: public gloss32
{
public:

  ppc_ppcbug();

  enum {
    BUGAPI_inchr	= 0x00,
    BUGAPI_inln		= 0x02,
    BUGAPI_readstr	= 0x03,
    BUGAPI_readln	= 0x04,

    BUGAPI_outchr	= 0x20,
    BUGAPI_outstr	= 0x21,
    BUGAPI_outln	= 0x22,

    BUGAPI_return	= 0x63,
    
    BUGAPI_time		= 0x70
  };

private:

  // ABI-specifics, for getting syscall arguments and setting results.
  // ??? Class by itself.
  // ??? Perhaps this shouldn't be here at all as it's really an
  // implementation detail (but lots of implementations will probably use it).
  bool get_int_argument(unsigned index, int32& value);
  bool set_int_result(int32 value);
  bool set_error_result(int32 value);

  // Return boolean indicating if cpu is requesting something we're to handle.
  bool syscall_trap_p();
  // Perform a system call trap.
  void syscall_trap();

  // Convert host errno to target errno.
  int host_to_target_errno (int errno_);
};

#endif /* PPC_H */
